unit Graphs;

interface

Uses
  DAutomatons;

  Procedure CreateMealyGraph(Var MAut:TMealyAutomaton; FileName:String);
  Procedure CreateMooreGraph(Var MAut:TMooreAutomaton; FileName:String);

Const
  StrAPrefix:String='a';
  StrZPrefix:String='z';
  StrWPrefix:String='w';
  FirstNodeColor:String='[color="blue"]';
  MealyFontColor:String='[labelfontcolor="red"]';
  MealyMinLen:String='[minlen=2]';
  MealyLabelDistance:String='[labeldistance=2]';
  MealyFontSize:String='[labelfontsize=12]';
  MooreFontColor:String='[fontcolor="red"]';

implementation
Uses
  SysUtils;

var
  F:Text;

Procedure CreateMealyGraph(Var MAut:TMealyAutomaton; FileName:String);
Var
  z, ACnt, ZCnt:Integer;
  FromA, ToA, W:TStateSet;
  FromAStr, ToAStr, ZStr, WStr:String;
  TmpStr:String;
begin
  AssignFile(F,Filename);
  Rewrite(F);

  ACnt:=MAut.GetACount;
  ZCnt:=MAut.GetZCount;

  Writeln(f,'digraph g{');

  For FromA:=1 To ACnt Do
  Begin
    FromAStr:=StrAPrefix+IntToStr(FromA);
    TmpStr:='  '+FromAStr+' [shape=circle]';
    If FromA=1 Then TmpStr:=TmpStr+' '+FirstNodeColor;
    TmpStr:=TmpStr+';';
    WriteLn(F, TmpStr);
    For z:=1 To ZCnt Do
    Begin
      ToA:=MAut.GetNextStateByState(FromA, z);
      W:=MAut.GetOutByState(FromA, z);

      If (ToA+W<>0) Then
      Begin
        ZStr:='[taillabel='+StrZPrefix+IntToStr(z)+']';
        If ToA=0 Then ToA:=FromA;
        ToAStr:=StrAPrefix+IntToStr(ToA);
        If W<>0 Then
          WStr:='[headlabel='+StrWPrefix+IntToStr(W)+']'
        Else
          WStr:='';
        TmpStr:='  '+FromAStr+' -> '+ToAStr+' '+WStr+' '+ZStr+' '+
          MealyFontColor+' '+MealyMinLen+' '+MealyLabelDistance+' '+MealyFontSize+';';
        WriteLn(F, TmpStr);
      End;
    End;
  End;

  Writeln(f,'}');

  CloseFile(f);
end;

Procedure CreateMooreGraph(Var MAut:TMooreAutomaton; FileName:String);
Var
  z, ACnt, ZCnt:Integer;
  FromA, ToA, W:TStateSet;
  FromAStr, ToAStr, ZStr, WStr:String;
  TmpStr:String;
begin
  AssignFile(F,Filename);
  Rewrite(F);

  ACnt:=MAut.GetACount;
  ZCnt:=MAut.GetZCount;

  Writeln(f,'digraph g{');

  For FromA:=1 To ACnt Do
  Begin
    FromAStr:=StrAPrefix+IntToStr(FromA);
    W:=MAut.GetOutByState(FromA, 0);
    If W<>0 Then
      WStr:=StrWPrefix+IntToStr(W)
    Else
      WStr:='--';

    TmpStr:='  '+FromAStr+' [shape=circle label="'+FromAStr+'\n'+WStr+'"]';
    If FromA=1 Then TmpStr:=TmpStr+' '+FirstNodeColor;
    TmpStr:=TmpStr+';';
    WriteLn(F, TmpStr);

    For z:=1 To ZCnt Do
    Begin
      ToA:=MAut.GetNextStateByState(FromA, z);
      If ToA<>0 Then
      Begin
        ToAStr:=StrAPrefix+IntToStr(ToA);
        ZStr:='[label='+StrZPrefix+IntToStr(z)+']';
        TmpStr:='  '+FromAStr+' -> '+ToAStr+' '+ZStr+' '+MooreFontColor+';';
        WriteLn(F, TmpStr);
      End;
    End;
  End;

  Writeln(F,'}');

  CloseFile(f);
end;

end.
